/**
 * Copyright (C) 2015 - 2018 Kosmos contact@kosmos.fr
 *
 * Projet: blog
 * Version: 1.08.00-BETA-00
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kosmos.blog.om;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Types;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.StringTokenizer;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.jsbsoft.jtf.core.Formateur;
import com.jsbsoft.jtf.database.OMContext;
import com.kosmos.blog.bean.ArticleBlogBean;
import com.kosmos.service.impl.ServiceManager;
import com.kportal.cms.objetspartages.annotation.FicheAnnotation;
import com.kportal.extension.module.plugin.objetspartages.PluginFicheHelper;
import com.kportal.extension.module.plugin.objetspartages.om.ObjetPluginContenu;
import com.univ.objetspartages.bean.MetatagBean;
import com.univ.objetspartages.bean.RubriqueBean;
import com.univ.objetspartages.om.AbstractFicheRestriction;
import com.univ.objetspartages.om.EtatFiche;
import com.univ.objetspartages.om.InfosRubriques;
import com.univ.objetspartages.om.StructureModele;
import com.univ.objetspartages.services.ServiceRubrique;
import com.univ.objetspartages.services.ServiceStructure;
import com.univ.objetspartages.util.MetatagUtils;
import com.univ.utils.ContexteDao;
import com.univ.utils.ContexteUtil;
import com.univ.utils.DateUtil;
import com.univ.utils.FicheUnivMgr;
import com.univ.utils.RequeteUtil;
import com.univ.utils.UnivWebFmt;
import com.univ.utils.sql.Operateur;
import com.univ.utils.sql.RequeteSQL;
import com.univ.utils.sql.clause.ClauseLimit;
import com.univ.utils.sql.clause.ClauseOrderBy;
import com.univ.utils.sql.clause.ClauseOrderBy.SensDeTri;
import com.univ.utils.sql.clause.ClauseWhere;
import com.univ.utils.sql.condition.Condition;
import com.univ.utils.sql.condition.ConditionList;
import com.univ.utils.sql.criterespecifique.ClauseJoinHelper;
import com.univ.utils.sql.criterespecifique.ConditionHelper;
import com.univ.utils.sql.criterespecifique.LimitHelper;
import com.univ.utils.sql.criterespecifique.RequeteSQLHelper;
import com.univ.utils.sql.operande.TypeOperande;

/**
 * Classe representant un objet articleblog.
 */
@FicheAnnotation(isEncadreRechercheEmbarquable = false, isEncadreRecherche = false)
public class Articleblog extends AbstractFicheRestriction<ArticleBlogBean> {

    private static final Logger LOGGER = LoggerFactory.getLogger(Articleblog.class);

    /**
     * Renvoie le libelle a afficher (methode statique utilisee pour les jointures entre fiches).
     * @deprecated ne devrait pas être utiliser, au pire des cas voir {@link Articleblog#getLibelleAffichable(String, String)}
     */
    @Deprecated
    public static String getLibelleAffichable(final OMContext ctx, final String codes, String langue) throws Exception {
        return getLibelleAffichable(codes, langue);
    }

    /**
     * Renvoie le libelle a afficher (methode statique utilisee pour les jointures entre fiches).
     */
    public static String getLibelleAffichable(final String codes, String langue) throws Exception {
        String res = StringUtils.EMPTY;
        if (codes == null || codes.length() == 0) {
            return res;
        }
        final Articleblog articleblog = new Articleblog();
        articleblog.init();
        if (langue == null || langue.length() == 0) {
            langue = "0";
        }
        final StringTokenizer st = new StringTokenizer(codes, ";");
        while (st.hasMoreTokens()) {
            final String code = st.nextToken();
            // On cherche d'abord la version en ligne puis les autres versions
            int count = articleblog.selectCodeLangueEtat(code, langue, EtatFiche.EN_LIGNE.getEtat());
            if (count == 0) {
                count = articleblog.selectCodeLangueEtat(code, langue, StringUtils.EMPTY);
            }
            if (count > 0) {
                if (res.length() > 1) {
                    res += ";";
                }
                if (articleblog.nextItem()) {
                    res += articleblog.getLibelleAffichable();
                }
            } else {
                res += "-";
            }
        }
        return res;
    }

    /**
     * Renvoie la fiche demandee (methode statique utilisee pour les jointures entre fiches).
     * @deprecated ne devrait pas être utiliser, au pire des cas voir {@link Articleblog#getFicheArticleblog(String, String)}
     */
    @Deprecated
    public static Articleblog getFicheArticleblog(final OMContext ctx, final String code, String langue) throws Exception {
        return getFicheArticleblog(code, langue);
    }

    /**
     * Renvoie la fiche demandee (methode statique utilisee pour les jointures entre fiches).
     */
    public static Articleblog getFicheArticleblog(final String code, String langue) throws Exception {
        Articleblog fiche = null;
        if (code != null && code.length() > 0) {
            if (langue == null || langue.length() == 0) {
                langue = "0"; // francais par defaut
            }
            fiche = new Articleblog();
            fiche.init();
            int count = fiche.selectCodeLangueEtat(code, langue, EtatFiche.EN_LIGNE.getEtat());
            if (count == 0) // si pas de version en ligne
            {
                count = fiche.selectCodeLangueEtat(code, langue, StringUtils.EMPTY);
            }
            if (count == 0) // si pas de version dans la langue demandee
            {
                count = fiche.selectCodeLangueEtat(code, StringUtils.EMPTY, EtatFiche.EN_LIGNE.getEtat());
            }
            if (count > 0) {
                fiche.nextItem();
            }
        }
        return fiche;
    }

    /**
     * Initialise l'objet metier.
     */
    @Override
    public void init() {
        super.init();
        setChapeau(StringUtils.EMPTY);
        setContenu(StringUtils.EMPTY);
        setKeywords(StringUtils.EMPTY);
    }

    /**
     * Renvoie le libelle a afficher.
     */
    @Override
    public String getLibelleAffichable() {
        return getServiceFiche().getLibelleAffichable(persistenceBean);
    }

    /**
     * Renvoie le contenu formate en HTML pour l'attribut contenu.
     */
    public String getFormatedContenu() throws Exception {
        return UnivWebFmt.formaterEnHTML(ContexteUtil.getContexteUniv(), getContenu());
    }

    /**
     * Renvoie la rubrique de la fiche.
     */
    public InfosRubriques getInfosRubrique() {
        InfosRubriques res = new InfosRubriques("");
        if (StringUtils.isNotBlank(getCodeRubrique())) {
            final ServiceRubrique serviceRubrique = ServiceManager.getServiceForBean(RubriqueBean.class);
            final RubriqueBean rubriqueBean = serviceRubrique.getRubriqueByCode(getCodeRubrique());
            if (rubriqueBean != null) {
                res = new InfosRubriques(rubriqueBean);
            }
        }
        return res;
    }

    /**
     * Renvoie la structure de la fiche.
     *
     * @deprecated Utiliser {@link com.univ.objetspartages.services.ServiceStructure#getByCodeLanguage(String, String)}
     */
    @Deprecated
    public StructureModele getInfosStructure() {
        final ServiceStructure serviceStructure = ServiceManager.getServiceForBean(StructureModele.class);
        return serviceStructure.getByCodeLanguage(getCodeRattachement(), getLangue());
    }

    /**
     * Renvoie les metadonnees de la fiche.
     */
    public MetatagBean getMeta() throws Exception {
        return MetatagUtils.lireMeta(this);
    }

    /**
     * Traitement d'une requete sur l'objet.
     */
    @Override
    public int traiterRequete(final String requete) throws Exception {
        // Recuperation des parametres de la requete
        final String titre = RequeteUtil.renvoyerParametre(requete, "TITRE");
        final String keywords = RequeteUtil.renvoyerParametre(requete, "KEYWORDS");
        final String codeRubrique = RequeteUtil.renvoyerParametre(requete, "CODE_RUBRIQUE");
        final String codeRattachement = RequeteUtil.renvoyerParametre(requete, "CODE_RATTACHEMENT");
        final String langue = RequeteUtil.renvoyerParametre(requete, "LANGUE");
        final String nombre = RequeteUtil.renvoyerParametre(requete, "NOMBRE");
        final String dateMiseEnLigneSaisie = RequeteUtil.renvoyerParametre(requete, "DATE_MISE_EN_LIGNE");
        Date dateMiseEnligneDebut = null;
        Date dateMiseEnligneFin = null;
        if (StringUtils.isNotBlank(dateMiseEnLigneSaisie)) {
            final Date dateMiseEnLigne = Formateur.parserDate(dateMiseEnLigneSaisie);
            if (Formateur.estSaisie(dateMiseEnLigne)) {
                final Calendar calendar = new GregorianCalendar();
                calendar.setTime(dateMiseEnLigne);
                calendar.set(Calendar.DAY_OF_MONTH, 1);
                dateMiseEnligneDebut = new java.sql.Date(calendar.getTimeInMillis());
                calendar.set(Calendar.DAY_OF_MONTH, 31);
                dateMiseEnligneFin = new java.sql.Date(calendar.getTimeInMillis());
            }
        }
        if (!Formateur.estSaisie(dateMiseEnligneDebut) && !Formateur.estSaisie(dateMiseEnligneFin)) {
            dateMiseEnligneDebut = DateUtil.parseDate(RequeteUtil.renvoyerParametre(requete, "DATE_DEBUT"));
            dateMiseEnligneFin = DateUtil.parseDate(RequeteUtil.renvoyerParametre(requete, "DATE_FIN"));
        }
        final String etatObjet = StringUtils.defaultIfBlank(RequeteUtil.renvoyerParametre(requete, "ETAT_OBJET"), EtatFiche.EN_LIGNE.toString());
        final String codeRedacteur = RequeteUtil.renvoyerParametre(requete, "CODE_REDACTEUR");
        final String code = RequeteUtil.renvoyerParametre(requete, "CODE");
        return select(titre, keywords, codeRubrique, codeRattachement, langue, etatObjet, codeRedacteur, code, nombre, dateMiseEnligneDebut, dateMiseEnligneFin);
    }

    /**
     * Renvoie la liste des references a cette fiche.
     */
    @Override
    public String getReferences() {
        return StringUtils.defaultString(getContenu()) + FicheUnivMgr.getReferenceParJointure("structure", getCodeRattachement());
    }

    /**
     * Selection d'une instance de l'objet Articleblog a partir de l'ensemble des criteres.
     */
    public int select(final String titre, final String keywords, final String codeRubrique, final String codeRattachement, final String langue, final String etatObjet, final String codeRedacteur, final String code, final String nombre, final Date dateMiseEnligneDebut, final Date dateMiseEnligneFin) throws Exception {
        final ClauseWhere where = new ClauseWhere();
        // Traitement du parametre titre
        // Recherche également sur le champs chapeau
        if (StringUtils.isNotEmpty(titre)) {
            final ConditionList listeConditions = new ConditionList();
            listeConditions.setPremiereCondtion(ConditionHelper.rechercheMots("T1.TITRE", titre));
            listeConditions.or(ConditionHelper.rechercheMots("T1.CHAPEAU", titre));
            where.setPremiereCondition(listeConditions);
        }
        if (StringUtils.isNotEmpty(keywords)) {
            where.and(ConditionHelper.rechercheMots("T1.KEYWORDS", keywords));
        }
        if (StringUtils.isNotEmpty(codeRattachement)) {
            where.and(ConditionHelper.getConditionStructure("T1.CODE_RATTACHEMENT", codeRattachement));
        }
        if (StringUtils.isNotEmpty(langue) && !"0000".equals(langue)) {
            where.and(ConditionHelper.egalVarchar("T1.LANGUE", langue));
        }
        if (StringUtils.isNotEmpty(etatObjet) && !"0000".equals(etatObjet)) {
            where.and(ConditionHelper.egalVarchar("T1.ETAT_OBJET", etatObjet));
        }
        if (StringUtils.isNotEmpty(codeRedacteur)) {
            where.and(ConditionHelper.egalVarchar("T1.CODE_REDACTEUR", codeRedacteur));
        }
        if (StringUtils.isNotEmpty(code)) {
            where.and(ConditionHelper.egalVarchar("T1.CODE", code));
        }
        // Traitement du parametre metaDateMiseEnLigneStart
        if (Formateur.estSaisie(dateMiseEnligneDebut)) {
            final Condition condition = ConditionHelper.genericCondition("SUBSTRING(META.META_DATE_MISE_EN_LIGNE, 1, 10)", TypeOperande.NON_ECHAPABLE, dateMiseEnligneDebut, TypeOperande.DATE, Operateur.GREATER_EQUALS);
            where.and(condition);
        }
        // Traitement du parametre metaDateMiseEnLigneEnd
        if (Formateur.estSaisie(dateMiseEnligneFin)) {
            final Condition condition = ConditionHelper.genericCondition("SUBSTRING(META.META_DATE_MISE_EN_LIGNE, 1, 10)", TypeOperande.NON_ECHAPABLE, dateMiseEnligneFin, TypeOperande.DATE, Operateur.LESS_EQUALS);
            where.and(condition);
        }
        final RequeteSQL requeteSelect = RequeteSQLHelper.getRequeteGenerique(where, ctx, this, codeRubrique);
        final ClauseOrderBy orderBy = new ClauseOrderBy();
        orderBy.orderBy("T1.TITRE", SensDeTri.ASC);
        requeteSelect.orderBy(orderBy);
        final ClauseLimit limite = LimitHelper.ajouterCriteresLimitesEtOptimisation(ctx, nombre);
        requeteSelect.limit(limite);
        // jointure METATAG
        if (Formateur.estSaisie(dateMiseEnligneDebut) || Formateur.estSaisie(dateMiseEnligneFin)) {
            requeteSelect.join(ClauseJoinHelper.creerJointureMetaTag(this));
        }
        return select(requeteSelect.formaterRequete());
    }

    public Blog getBlog() throws Exception {
        Blog blog = null;
        try (ContexteDao ctx = new ContexteDao();
             Connection connection = ctx.getConnection();
             PreparedStatement stmt = connection.prepareStatement("select ID_BLOG from BLOG where ETAT_OBJET = ? and CODE = ?")) {
            stmt.setObject(1, "0003", Types.VARCHAR);
            stmt.setObject(2, getCodeRubrique(), Types.VARCHAR);
            try (ResultSet rs = stmt.executeQuery()) {
                if (rs.next()) {
                    blog = new Blog();
                    blog.init();
                    blog.setIdBlog(rs.getLong("ID_BLOG"));
                    blog.retrieve();
                }
            }
        }
        return blog;
    }

    /**
     * @deprecated cette méthode ne devrait pas être placer sur cet objet, elle ne devrait pas être une méthode d'instance, etc etc.
     * @return
     */
    @Deprecated
    public int getNombreReponses() {
        int nbCommentaires = 0;
        try {
            final MetatagBean meta = MetatagUtils.lireMeta(this);
            final List<ObjetPluginContenu> listeCommentaires = PluginFicheHelper.lireObjets(this, meta, "com.kosmos.commentaire.om.Commentaire");
            if (listeCommentaires != null) {
                nbCommentaires = listeCommentaires.size();
            }
        } catch (final Exception e) {
            LOGGER.error("impossible de récupérer le nombre de réponse", e);
        }
        return nbCommentaires;
    }

    public Long getIdArticleblog() {
        return persistenceBean.getId();
    }

    public void setIdArticleblog(Long id) {
        persistenceBean.setId(id);
    }

    public String getChapeau() {
        return persistenceBean.getChapeau();
    }

    public void setChapeau(String chapeau) {
        persistenceBean.setChapeau(chapeau);
    }

    public String getContenu() {
        return persistenceBean.getContenu();
    }

    public void setContenu(String contenu) {
        persistenceBean.setContenu(contenu);
    }

    public String getKeywords() {
        return persistenceBean.getKeywords();
    }

    public void setKeywords(String keywords) {
        persistenceBean.setKeywords(keywords);
    }
}
